__author__ = 'DarkWeb'

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup


# parses description pages, so takes html pages of description pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of description page
# return: 'row' that contains a variety of lists that each hold info on the description page
def metaversemarket_description_parser(soup):
    # Fields to be parsed

    name = "-1"  # 0 Product_Name
    describe = "-1"  # 1 Product_Description
    lastSeen = "-1"  # 2 Product_LastViewDate
    CVE = "-1"  # 4 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"  # 5 Product_MS_Classification (Microsoft Security)
    review = "-1"  # 6 Product_Number_Of_Reviews
    category = "-1"  # 7 Product_Category
    shipFrom = "-1"  # 8 Product_ShippedFrom
    shipTo = "-1"  # 9 Product_ShippedTo
    left = "-1"  # 10 Product_QuantityLeft
    escrow = "-1"  # 11 Vendor_Warranty
    terms = "-1"  # 12 Vendor_TermsAndConditions
    vendor = "-1"  # 13 Vendor_Name
    sold = "-1"  # 14 Product_QuantitySold
    addDate = "-1"  # 15 Product_AddedDate
    BTC = "-1"  # 18 Product_BTC_SellingPrice
    USD = "-1"  # 19 Product_USD_SellingPrice
    rating = "-1"  # 20 Vendor_Rating
    success = "-1"  # 21 Vendor_Successful_Transactions
    EURO = "-1"  # 22 Product_EURO_SellingPrice

    # Finding Product Name
    name = soup.find('div', {'class': "panel-heading"}).text.strip

    # Finding Vendor
    temp = soup.findAll('div', {'class': "col-xs-12 col-sm-6 mt-5"})
    temp = temp[1].findAll('span')
    temp = temp[1].find('b').text
    name = temp.replace("@", "")

    # Finding Product Reviews
    review = soup.find('span', {'class': "badge bg-success fs-12px"}).text.strip()

    # Finding Successful Transactions
    # NA


    # Finding Prices
    USD = soup.find('h3', {'class': "mb-2"}).text()
    USD = USD.replace("Price: $", "").strip()

    # Finding Escrow
    escrow = soup.find('div', {'class': "alert alert-info text-center fw-bold"}).text
    escrow = escrow.replace('You are protected by ', "").strip()

    # Finding the Product Category
    temp = soup.select('div[class="mt-2"]')
    temp = temp[0].findAll('span')
    category = temp[1].text.strip()

    # Finding the Product Quantity Available
    # temp = soup.find('em', {'class': "icon ni ni-layers-fill"}).parent.parent.parent
    # left = temp.text
    # left = left.replace("Supply:", "")
    # left = left.strip()
    temp = soup.findAll('span', {'class': "badge bg-success"})
    temp = temp[1].text.split("/")
    left = temp[1].strip()

    # Finding Number Sold
    sold = temp[0].strip()


    # Finding Shipment Information (Origin)
    temp = soup.findAll('div', {'class': "alert alert-info"})
    temp = temp[1].text.split("to")
    shipFrom = temp[0].replace("Shipping from ", "").strip()

    # Finding Shipment Information (Destination)
    shipTo = temp[1].split("for")
    shipTo = shipTo[0].strip()

    # Finding the Product description
    describe = soup.find('p', {'class': "card-text"}).text
    describe = describe.replace("\n", " ")
    describe = describe.strip()

    '''# Finding the Number of Product Reviews
    tag = soup.findAll(text=re.compile('Reviews'))
    for index in tag:
        reviews = index
        par = reviews.find('(')
        if par >=0:
            reviews = reviews.replace("Reviews (","")
            reviews = reviews.replace(")","")
            reviews = reviews.split(",")
            review = str(abs(int(reviews[0])) + abs(int(reviews[1])))
        else  :
            review = "-1"'''

    # Searching for CVE and MS categories
    cve = soup.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
    if cve:
        CVE = " "
        for idx in cve:
            CVE += (idx)
            CVE += "  "
            CVE = CVE.replace(',', ' ')
            CVE = CVE.replace('\n', '')
    ms = soup.findAll(text=re.compile('MS\d{2}-\d{3}'))
    if ms:
        MS = " "
        for im in ms:
            MS += (im)
            MS += " "
            MS = MS.replace(',', ' ')
            MS = MS.replace('\n', '')

    # Populating the final variable (this should be a list with all fields scraped)
    row = (name, describe, lastSeen, CVE, MS, review, category, shipFrom, shipTo, left, escrow, terms, vendor,
           sold, addDate, BTC, USD, rating, success, EURO)

    # Sending the results
    return row


# parses listing pages, so takes html pages of listing pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of listing page
# return: 'row' that contains a variety of lists that each hold info on the listing page
def metaversemarket_listing_parser(soup):
    # Fields to be parsed
    nm = 0  # Total_Products (Should be Integer)
    mktName = "DarkFox"  # 0 Marketplace_Name
    name = []  # 1 Product_Name
    CVE = []  # 2 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = []  # 3 Product_MS_Classification (Microsoft Security)
    category = []  # 4 Product_Category
    describe = []  # 5 Product_Description
    escrow = []  # 6 Vendor_Warranty
    views = []  # 7 Product_Number_Of_Views
    reviews = []  # 8 Product_Number_Of_Reviews
    addDate = []  # 9 Product_AddDate
    lastSeen = []  # 10 Product_LastViewDate
    BTC = []  # 11 Product_BTC_SellingPrice
    USD = []  # 12 Product_USD_SellingPrice
    EURO = []  # 13 Product_EURO_SellingPrice
    sold = []  # 14 Product_QuantitySold
    qLeft = []  # 15 Product_QuantityLeft
    shipFrom = []  # 16 Product_ShippedFrom
    shipTo = []  # 17 Product_ShippedTo
    vendor = []  # 18 Vendor
    rating = []  # 19 Vendor_Rating
    success = []  # 20 Vendor_Successful_Transactions
    href = []  # 23 Product_Links (Urls)

    listing = soup.findAll('div', {"class": "col-12 col-sm-4 col-xl-3 product_item_col p-1"})

    # Populating the Number of Products
    nm = len(listing)

    for a in listing:
        bae = a.findAll('a', href=True)

        # Adding the url to the list of urls
        link = bae[0].get('href')
        link = cleanLink(link)
        href.append(link)

        # Finding the Product
        product = bae[1].find('span', {"class": "text-primary"}).text
        product = product.replace('\n', ' ')
        product = product.replace(",", "")
        product = product.replace("...", "")
        product = product.strip()
        name.append(product)

        # Finding Prices
        price = a.find('strong').text
        price = price.replace("Buy for $", "")
        price = price.strip()
        USD.append(price)


        # Finding the Vendor
        temp = a.find('div', {'class': "mt-1 fs-12px"})
        temp = temp.findAll('span')
        temp = temp[1].find('b').text
        vendor_name = temp.replace("@", "").strip()
        vendor.append(vendor_name)

        # Finding the Category
        cat = a.select_one('div[class="fs-12px"]')
        cat = cat.findAll('span')[1].text
        cat = cat.text
        cat = cat.strip()
        category.append(cat)

        badge = a.findAll('span', {'class': "badge bg-success"})
        # Finding Number Sold and Quantity Left
        temp = badge[1].text
        temp = temp.split("/")
        num = temp[0]
        num = num.strip()
        sold.append(num)

        quant = temp[1]
        quant = quant.strip()
        qLeft.append(quant)

        # Finding Successful Transactions
        # NA

        # Finding Product review
        review = a.find('span', {'class': "badge bg-success fs-12px"}).text
        review = review.replace("+ ", "")
        reviews.append(review)

        # Finding Descrption
        description = a.find('p', {'class': "alert alert-light text-ssbold p-1"}).text
        description = description.replace("\n", " ")
        description = description.strip()
        describe.append(description)

        # Finding Escrow
        es = a.find('span', {'class': "fw-bold"}).text.strip()
        escrow.append(es)

        # Finding Number of Views
        view = a.find('span', {'class': "badge bg-primary"}).text.strip()
        views.append(view)

        # Find where ships from
        ships = a.find('div', {'class': "alert alert-info item_alert fs-12px p-1"})
        ships = ships.findAll('b')
        sFrom = ships[0].text.strips()
        shipFrom.append(sFrom)

        # Find where it ships to
        sTo = ships[1].text.strips()
        shipTo.append(sTo)

        # Searching for CVE and MS categories
        cve = a.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
        if not cve:
            cveValue = "-1"
        else:
            cee = " "
            for idx in cve:
                cee += (idx)
                cee += "  "
                cee = cee.replace(',', ' ')
                cee = cee.replace('\n', '')
            cveValue = cee
        CVE.append(cveValue)

        ms = a.findAll(text=re.compile('MS\d{2}-\d{3}'))
        if not ms:
            MSValue = "-1"
        else:
            me = " "
            for im in ms:
                me += (im)
                me += " "
                me = me.replace(',', ' ')
                me = me.replace('\n', '')
            MSValue = me
        MS.append(MSValue)

    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(mktName, nm, name, CVE, MS, category, describe, escrow, views, reviews, addDate, lastSeen,
                            BTC, USD, EURO, qLeft, shipFrom, shipTo, vendor, rating, success, sold, href)


# called by the crawler to get description links on a listing page
# @param: beautifulsoup object that is using the correct html page (listing page)
# return: list of description links from a listing page
def metaversemarket_links_parser(soup):
    # Returning all links that should be visited by the Crawler

    href = []
    listing = soup.findAll('div', {"class": "col-12 col-sm-4 col-xl-3 product_item_col p-1"})

    for a in listing:
        bae = a.find('a', href=True)
        link = bae['href']
        href.append(link)

    return href